@@ -8,8 +8,7 @@ selection method.
 
 """
 
-#
-# Copyright (c) 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008 The SCons Foundation
+# Copyright (c) 2001 - 2015 The SCons Foundation
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -29,9 +28,8 @@ selection method.
 # LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
-#
 
-__revision__ = "src/engine/SCons/Tool/Perforce.py 3363 2008/09/06 07:34:10 scons"
+__revision__ = "src/engine/SCons/Tool/Perforce.py rel_2.3.5:3329:275e75118ad4 2015/06/20 11:18:26 bdbaddog"
 
 import os
 
@@ -44,10 +42,9 @@ import SCons.Util
 from SCons.Tool.PharLapCommon import addPathIfNotExists
 
 
-
 # Variables that we want to import from the base OS environment.
 _import_env = [ 'P4PORT', 'P4CLIENT', 'P4USER', 'USER', 'USERNAME', 'P4PASSWD',
-                'P4CHARSET', 'P4LANGUAGE', 'SYSTEMROOT' ]
+                'P4CHARSET', 'P4LANGUAGE', 'SystemRoot' ]
 
 PerforceAction = SCons.Action.Action('$P4COM', '$P4COMSTR')
 
@@ -57,6 +54,8 @@ def generate(env):
 
     def PerforceFactory(env=env):
         """ """
+        import SCons.Warnings as W
+        W.warn(W.DeprecatedSourceCodeWarning, """The Perforce() factory is deprecated and there is no replacement.""")
         return SCons.Builder.Builder(action = PerforceAction, env = env)
 
     #setattr(env, 'Perforce', PerforceFactory)
@@ -96,3 +95,9 @@ def generate(env):
 
 def exists(env):
     return env.Detect('p4')
+
+# Local Variables:
+# tab-width:4
+# indent-tabs-mode:nil
+# End:
+# vim: set expandtab tabstop=4 shiftwidth=4:
